import logging

from celery import shared_task
from django.contrib.auth.models import User
from .hooks import ServicesHook
from .celery_once.tasks import QueueOnce  # noqa: F401 - for backwards compatibility

logger = logging.getLogger(__name__)


@shared_task(bind=True)
def validate_services(self, pk):
    user = User.objects.get(pk=pk)
    logger.debug(f'Ensuring user {user} has permissions for active services')
    # Iterate through services hooks and have them check the validity of the user
    for svc in ServicesHook.get_services():
        try:
            svc.validate_user(user)
        except Exception:
            logger.exception(f'Exception running validate_user for services module {svc} on user {user}')


def disable_user(user):
    logger.debug('Disabling all services for user %s' % user)
    for svc in ServicesHook.get_services():
        if svc.service_active_for_user(user):
            svc.delete_user(user)


@shared_task
def update_groups_for_user(user_pk: int) -> None:
    """Update groups for all services registered to a user."""
    user = User.objects.get(pk=user_pk)
    logger.debug("%s: Triggering service group update for user", user)
    for svc in ServicesHook.get_services():
        try:
            svc.validate_user(user)
            svc.update_groups(user)
        except Exception:
            logger.exception(
                'Exception running update_groups for services module %s on user %s',
                svc,
                user
            )
